/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.base;

import java.util.Collections;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.interfaces.IModelElementHandler;

/**
 * Base implementation for {@link IModelElementHandler}s.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @levd.rating RED Rev:
 */
public abstract class ModelElementHandlerBase<T extends EObject> implements
		IModelElementHandler<T> {

	/** Stores the singleton of the icon image. */
	private Image iconImage;

	/** {@inheritDoc} */
	@Override
	public String getDescription(T element) {
		return "";
	}

	/** {@inheritDoc} */
	@Override
	public final Image getIcon(T element) {
		if (iconImage == null) {
			ImageDescriptor descr = getImageDescriptor();
			if (descr != null) {
				iconImage = descr.createImage();
			}
		}
		return iconImage;
	}

	/** Returns image descriptor to be used as icon image. */
	protected ImageDescriptor getImageDescriptor() {
		return ToolingKernelActivator.getImageDescriptor("icons/unknown.png");
	}

	/** {@inheritDoc} */
	@Override
	public List<EObject> getSubnodes(T element) {
		return Collections.emptyList();
	}

	/** {@inheritDoc} */
	@Override
	public List<EObject> getConnectors(T element) {
		return Collections.emptyList();
	}

	/** {@inheritDoc} */
	@Override
	public List<EObject> getIncomingConnections(T element) {
		return Collections.emptyList();
	}

	/** {@inheritDoc} */
	@Override
	public List<EObject> getOutgoingConnections(T element) {
		return Collections.emptyList();
	}

	/** {@inheritDoc} */
	@Override
	public List<EObject> getSpecifications(T element) {
		return Collections.emptyList();
	}
}
