/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import java.util.HashMap;
import java.util.Map;

import org.conqat.ide.commons.ui.extension.ExtensionPointUtils;
import org.conqat.ide.commons.ui.logging.LoggingUtils;
import org.conqat.lib.commons.reflect.ReflectionUtils;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.interfaces.IModelElementHandler;
import org.fortiss.tooling.kernel.services.IModelElementService;
import org.osgi.framework.Bundle;

/**
 * This class implements the {@link IModelElementService} interface.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @levd.rating RED Rev:
 */
public class ModelElementService implements IModelElementService {

	/** The model element handler extension point ID. */
	private static final String HANDLER_EXTENSION_POINT_NAME = "org.fortiss.tooling.kernel.modelElementHandler";

	/** The model element handler configuration element name. */
	private static final String HANDLER_CONFIGURATION_ELEMENT_NAME = "modelElementHandler";

	/** Stores the model element handler for each model element class. */
	private Map<Class<?>, IModelElementHandler<EObject>> handlerMap;

	/** Constructor. */
	public ModelElementService() {
		setupHandlerMap();
	}

	/** Initializes the handler map from plugin extensions. */
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void setupHandlerMap() {
		handlerMap = new HashMap<Class<?>, IModelElementHandler<EObject>>();
		for (IConfigurationElement ce : ExtensionPointUtils
				.getConfigurationElements(HANDLER_EXTENSION_POINT_NAME,
						HANDLER_CONFIGURATION_ELEMENT_NAME)) {
			Bundle bundle = ExtensionPointUtils.getBundle(ce);
			try {
				Class<?> modelElementClass = ExtensionPointUtils.loadClass(
						ce.getAttribute("modelElementClass"), bundle);
				Class<?> handlerClass = ExtensionPointUtils.loadClass(
						ce.getAttribute("handler"), bundle);
				IModelElementHandler<EObject> handler = (IModelElementHandler) handlerClass
						.getConstructor().newInstance();
				handlerMap.put(modelElementClass, handler);
			} catch (Exception ex) {
				LoggingUtils.error(ToolingKernelActivator.getDefault(),
						ex.getMessage(), ex);
			}
		}

	}

	/** {@inheritDoc} */
	@Override
	public IModelElementHandler<EObject> getModelElementHandler(
			EObject modelElement) {
		Class<? extends EObject> clazz = modelElement.getClass();
		IModelElementHandler<EObject> handler = handlerMap.get(clazz);
		if (handler == null) {
			handler = ReflectionUtils.performNearestClassLookup(clazz,
					handlerMap);
			if (handler == null) {
				LoggingUtils.error(ToolingKernelActivator.getDefault(),
						"No matching model element handler found for " + clazz,
						null);
			}
		}
		return handler;
	}
}
